/*
 * Copyright (C) 2008-2010, Aspire
 * 
 * Aspire is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License version 2.1 as published by
 * the Free Software Foundation (the "LGPL").
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library in the file COPYING-LGPL-2.1; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * 
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied. See the GNU Lesser General Public License
 * for the specific language governing rights and limitations.
 */

package org.inria.pops.application;

/**
 * @author Lei Zhang e-mail: lzhang@inria.fr
 *
 */

/**
 * The light ALECC aims to supply an easy and light framework to develop the most used basic operation defined by the EPC standard.
 * It can be flexibly inserted in your own applications.
 * 
 * For developers: (How to use)
 * This tool depends on the fosstrak jar.
 *  1) Firstly, you need to initialize LRSpec function:
 *  LRspec.initialize();
 *  2) Secondly, you need to initialize CCSpec function:
 *  CCspec.initialize();
 *  3) Thirdly, you need to define LRSpec: 
 *  LRspec.DefineLRSpec(LRname, LRSpecfilePath};
 *  4) Lastly, you need to define CCSpec and define the subscribe listening IP and port
 *  CCspec.DefineCCSpec(CCname, CCSpecfilePath,port ,IP);
 *  5) You can listen the report on the defined (IP:port) by the following thread RcvCCreportThread() defined in the demo
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.UIManager;
import javax.xml.bind.JAXBException;

import org.fosstrak.ale.xsd.ale.epcglobal.CCCmdReport;
import org.fosstrak.ale.xsd.ale.epcglobal.CCOpReport;
import org.fosstrak.ale.xsd.ale.epcglobal.CCReports;
import org.fosstrak.ale.xsd.ale.epcglobal.CCSpec;
import org.fosstrak.ale.xsd.ale.epcglobal.CCTagReport;
import org.fosstrak.ale.util.DeserializerUtil;
import org.fosstrak.ale.wsdl.ale.epcglobal.ImplementationExceptionResponse;
import org.fosstrak.ale.xsd.ale.epcglobal.ECSpec;
import org.fosstrak.ale.xsd.ale.epcglobal.LRSpec;
import org.fosstrak.ale.xsd.ale.epcglobal.CCCmdReport.TagReports;
import org.fosstrak.ale.xsd.ale.epcglobal.CCTagReport.OpReports;
import org.inria.pops.alecc.server.CCspec;
import org.inria.pops.alecc.server.LRspec;

/**
 * @author Lei Zhang e-mail: lzhang@inria.fr
 * 
 */

public class demo extends JPanel {

	static demo APP;
	Frame frame;
	Thread Listen_RCReport;

	JComboBox LRspecComboBox;
	JComboBox SpecNameComboBox;
	JComboBox CCSpecNameComboBox;
	final JTextField LRSpecfilePathField = new JTextField();
	final JTextField ECSpecfilePathField = new JTextField();
	final JTextField CCSpecfilePathField = new JTextField();
	final JTextField PortField = new JTextField();
	final JTextField IPField = new JTextField();

	boolean CCmode = false;

	int port;
	ServerSocket server_TDT_Socket;

	String data;
	Thread RcvCCreport_Thread = new RcvCCreportThread();

	JTextArea TextArea = new JTextArea("", 500, 500);
	JTextArea TextArea_EPC = new JTextArea("", 500, 300);

	public demo() throws IOException {
		port = 9999;
		server_TDT_Socket = new ServerSocket(port);

		TextArea.setForeground(Color.BLUE);
		TextArea.setFont(new java.awt.Font("Arial", Font.BOLD, 13));

		TextArea_EPC.setForeground(Color.BLUE);
		TextArea_EPC.setFont(new java.awt.Font("Arial", Font.BOLD, 13));

		JPanel MainPanel = new JPanel(new BorderLayout());
		MainPanel.setPreferredSize(new Dimension(1050, 730));
		// MainPanel.add("North", bar);
		MainPanel.add(Part());
		add(MainPanel);

	}

	protected JComponent Part() {

		JPanel Panel = new JPanel();
		Panel.setLayout(new BoxLayout(Panel, BoxLayout.X_AXIS));
		Panel.add(LPart());
		Panel.add(RPart());

		return Panel;

	}

	protected JComponent LPart() {

		JPanel Panel = new JPanel();
		Panel.setLayout(new BoxLayout(Panel, BoxLayout.Y_AXIS));
		int JTextField_line = 1;
		final int GAP = 10;

		JPanel Panel_app = new JPanel();
		Panel_app.setBorder(BorderFactory.createTitledBorder("Application"));
		JLabel label_app = new JLabel("YOUR APPLICATION HERE");
		Panel_app.add(label_app);

		JPanel Panel_1_outside = new JPanel() {
			// Don't allow us to stretch vertically.
			public Dimension getMaximumSize() {
				Dimension pref = getPreferredSize();
				return new Dimension(Integer.MAX_VALUE, pref.height);
			}
		};
		// Means the composants (sous-panel,Panel_1) to be added is on the LEFT
		// side of the panel "Panel_1_outside".
		Panel_1_outside.setLayout(new FlowLayout(FlowLayout.LEFT));
		JPanel Panel_1 = new JPanel();
		Panel_1.setLayout(new BoxLayout(Panel_1, BoxLayout.PAGE_AXIS));
		Panel_1_outside.setBorder(BorderFactory
				.createTitledBorder("Aspire ALECC"));

		// /Panel for the textfield///
		JPanel Panel_1_1 = new JPanel(new SpringLayout());

		// Panel_1.setLayout(null);

		// /////////////Panel for Textfield///////////////
		JLabel label_LRName = new JLabel("LR Name:");
		// final JTextField LRName = new JTextField();
		// LRName.setColumns(20);
		// label_LRName.setLabelFor(LRName);
		LRspecComboBox = new JComboBox();
		LRspecComboBox.setEditable(true);
		LRspecComboBox.addItem(null);
		LRspecComboBox.getEditor().setItem("ReaderInput");
		// /////////////////

		JLabel label_LRSpec = new JLabel("LRSpec:");

		LRSpecfilePathField.setColumns(20);
		LRSpecfilePathField.setText("C://LRSpecReaderInputCC.xml");
		final FileDialog fileDialog = new FileDialog(frame);
		fileDialog.setModal(true);
		fileDialog.addComponentListener(new ComponentAdapter() {
			public void componentHidden(ComponentEvent e) {
				if (fileDialog.getFile() != null) {
					LRSpecfilePathField.setText(fileDialog.getDirectory()
							+ fileDialog.getFile());
				}
			}
		});

		final JButton chooseFileButton = new JButton("File");
		chooseFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileDialog.setVisible(true);
			}
		});

		// /////////////////////////

		final JButton Define = new JButton("Define");
		Define.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String LRname = LRspecComboBox.getEditor().getItem().toString();
				LRspecComboBox.addItem(LRname);
				LRSpec spec = null;
				try {
					LRspec.DefineLRSpec(LRname, LRSpecfilePathField.getText());
				} catch (Exception e1) {
					System.err.println("Error Define LRSpec");
				}
				LRspecComboBox.getEditor().setItem("");
				// LRSpecfilePathField.setText("");

			}
		});

		final JButton UnDefine = new JButton("Undefine");
		UnDefine.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				String LRname = LRspecComboBox.getEditor().getItem().toString();
				LRspecComboBox.removeItem(LRname);
				try {
					LRspec.UnDefineLRSpec(LRname);
				} catch (Exception e2) {
					System.err.println("Error Undefine LRSpec");
				}
				LRspecComboBox.getEditor().setItem("");
				LRSpecfilePathField.setText("");
			}
		});

		JLabel Vain1 = new JLabel(" ");
		JLabel Vain2 = new JLabel(" ");
		JLabel Vain3 = new JLabel(" ");

		// /////////////Panel for Textfield///////////////
		JLabel label_CCSpecName = new JLabel("CC Spec Name:");
		// final JTextField SpecName = new JTextField();
		// SpecName.setColumns(20);
		// label_SpecName.setLabelFor(SpecName);
		CCSpecNameComboBox = new JComboBox();
		CCSpecNameComboBox.setEditable(true);
		CCSpecNameComboBox.addItem(null);
		CCSpecNameComboBox.getEditor().setItem("CCTagsys");

		// /////////////////

		JLabel label_CCSpec = new JLabel("CCSpec:");

		CCSpecfilePathField.setColumns(20);
		CCSpecfilePathField.setText("C://CCSpecReaderInput.xml");

		final FileDialog fileDialog3 = new FileDialog(frame);
		fileDialog3.setModal(true);
		fileDialog3.addComponentListener(new ComponentAdapter() {
			public void componentHidden(ComponentEvent e) {
				if (fileDialog3.getFile() != null) {
					CCSpecfilePathField.setText(fileDialog3.getDirectory()
							+ fileDialog3.getFile());
				}
			}
		});

		final JButton chooseFileButton3 = new JButton("File");
		chooseFileButton3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileDialog3.setVisible(true);
			}
		});

		// /////////////////////////

		JLabel label_CCReportsPort = new JLabel("Subscribe Port:");
		JLabel label_CCReportsIP = new JLabel("Subscribe IP:");

		PortField.setColumns(20);
		PortField.setText("9999");

		IPField.setColumns(20);
		IPField.setText("localhost");

		// /////////////////////////

		final JButton Define3 = new JButton("Define");
		Define3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String CCname = CCSpecNameComboBox.getEditor().getItem()
						.toString();
				CCSpecNameComboBox.addItem(CCname);
				CCSpec ccSpec = null;

				// get ecSpec
				try {

					CCspec.DefineCCSpec(CCname, CCSpecfilePathField.getText(),
							PortField.getText(), IPField.getText());
					CCmode = true;
				} catch (Exception e3) {
					System.err.println("Error Define CCSpec");
					CCmode = false;
				}

				CCSpecNameComboBox.getEditor().setItem("");
				CCSpecfilePathField.setText("");
			}
		});

		final JButton UnDefine3 = new JButton("Undefine");
		UnDefine3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String CCname = CCSpecNameComboBox.getEditor().getItem()
						.toString();
				CCSpecNameComboBox.removeItem(CCname);
				try {
					CCspec.UnDefineCCSpec(CCname);
					CCmode = false;
					TextArea_EPC.setText("");
					// data = "";
					// TextArea.setText("");
				} catch (Exception e1) {
					System.err.println("Error Undefine CCSpec");
					CCmode = false;
				}
				CCSpecNameComboBox.getEditor().setItem("");
			}
		});

		JLabel Vain31 = new JLabel(" ");
		JLabel Vain32 = new JLabel(" ");
		JLabel Vain33 = new JLabel(" ");
		JLabel Vain34 = new JLabel(" ");

		// /////////////////////////////////////
		Panel_1_1.add(label_LRName);
		Panel_1_1.add(LRspecComboBox);

		Panel_1_1.add(label_LRSpec);
		Panel_1_1.add(LRSpecfilePathField);
		Panel_1_1.add(chooseFileButton);

		Panel_1_1.add(Vain1);
		Panel_1_1.add(Define);
		Panel_1_1.add(Vain2);
		Panel_1_1.add(UnDefine);
		Panel_1_1.add(Vain3);

		// /
		Panel_1_1.add(label_CCSpecName);
		Panel_1_1.add(CCSpecNameComboBox);

		Panel_1_1.add(label_CCSpec);
		Panel_1_1.add(CCSpecfilePathField);
		Panel_1_1.add(chooseFileButton3);

		Panel_1_1.add(label_CCReportsPort);
		Panel_1_1.add(PortField);

		Panel_1_1.add(label_CCReportsIP);
		Panel_1_1.add(IPField);
		Panel_1_1.add(Vain34);

		Panel_1_1.add(Vain31);
		Panel_1_1.add(Define3);
		Panel_1_1.add(Vain32);
		Panel_1_1.add(UnDefine3);
		Panel_1_1.add(Vain33);

		SpringUtilities.makeCompactGrid(Panel_1_1, 5, 5, GAP, GAP, // init x,y
				GAP, GAP / 2);// xpad, ypad

		Panel_1.add(Panel_1_1);
		// Panel_1.add(Panel_1_2);
		Panel_1_outside.add(Panel_1);

		Panel.add(Panel_app);
		Panel.add(Panel_1_outside);

		return Panel;
	}

	protected JComponent RPart() {

		JPanel panel = new JPanel(new BorderLayout());
		panel.setPreferredSize(new Dimension(380, 800));

		JPanel Panel_3_outside = new JPanel() {
			public Dimension getMaximumSize() {
				Dimension pref = getPreferredSize();
				return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
			}
		};
		Panel_3_outside.setLayout(new FlowLayout(FlowLayout.LEFT));
		JScrollPane Panel_3 = new JScrollPane(TextArea,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		Panel_3.setPreferredSize(new Dimension(380, 450));
		Panel_3_outside.add(Panel_3);
		Panel_3_outside.setBorder(BorderFactory
				.createTitledBorder("ALE CCReport (Listening on 9999)"));

		JPanel Panel_5_outside = new JPanel() {
			public Dimension getMaximumSize() {
				Dimension pref = getPreferredSize();
				return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
			}
		};
		Panel_5_outside.setLayout(new FlowLayout(FlowLayout.LEFT));
		JScrollPane Panel_5 = new JScrollPane(TextArea_EPC,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		Panel_5.setPreferredSize(new Dimension(380, 200));
		Panel_5_outside.add(Panel_5);
		Panel_5_outside.setBorder(BorderFactory
				.createTitledBorder("Operation results: (Listening on 9999)"));

		panel.add("South", Panel_3_outside);
		panel.add("North", Panel_5_outside);

		return panel;

	}

	class RcvCCreportThread extends Thread {
		String oneline = null;

		public void run() {
			try {
				while (true) {
					String xmlline = "";
					java.net.Socket server_TDT = server_TDT_Socket.accept();
					BufferedReader in = new BufferedReader(
							new InputStreamReader(server_TDT.getInputStream()));

					while ((oneline = in.readLine()) != null) {
						data = data + oneline + "\n";
						xmlline = xmlline + oneline;
					}

					in.close();

					TextArea.setText(data);

					try {
						InputStream parseStream = new ByteArrayInputStream(
								xmlline.getBytes());
						TextArea_EPC.setText("");

						CCReports ccreports = DeserializerUtil
								.deserializeCCReports(parseStream);
						if (ccreports != null) {
							handleCCReports(ccreports);
						}

					} catch (Exception e) {
						System.err.println("Deserialization problem.");
					}
				}
			} catch (Exception e) {
				System.err.println("TCP socket problem.");
			}
		}
	}

	private void handleCCReports(CCReports reports) throws IOException,
			JAXBException {
		String DisplayValue = " ALECC Operation results: ";
		String TagID = "";
		String WrittenData = "";
		String opStatus = "";

		try {
			List<CCCmdReport> theReports = reports.getCmdReports()
					.getCmdReport();
			if (theReports != null) {
				DisplayValue = DisplayValue + "\n";
				for (CCCmdReport cmdreport : theReports) {
					TagReports TR = cmdreport.getTagReports();
					List<CCTagReport> TagReports = TR.getTagReport();
					for (CCTagReport Tagreport : TagReports) {
						DisplayValue = DisplayValue + "\n";
						TagID = Tagreport.getId();
						DisplayValue = DisplayValue + "Tag ID: " + TagID + "\n";
						OpReports OpReports = Tagreport.getOpReports();
						List<CCOpReport> OReports = OpReports.getOpReport();
						for (CCOpReport OReport : OReports) {
							WrittenData = OReport.getData();
							DisplayValue = DisplayValue + "Writtrn Data: "
									+ WrittenData + "\n";
							opStatus = OReport.getOpStatus();
							DisplayValue = DisplayValue + "Operation Status: "
									+ opStatus + "\n";
						}
					}

				}
			}

			TextArea_EPC.setText(DisplayValue);

		} catch (Exception e) {
			System.err.println("CCReports Deserialization problem.");
		}

	}

	private static void createAndShowGUI() {

		JFrame frame = new JFrame("Aspire ALE Demo");
		frame.setContentPane(APP);

		frame.setResizable(false);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}

	public static void main(String[] args)
			throws ImplementationExceptionResponse, IOException {

		try {
			UIManager
					.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Exception e) {
		}

		APP = new demo();
		APP.RcvCCreport_Thread.start();

		JFrame.setDefaultLookAndFeelDecorated(true);

		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();

			}
		});

		LRspec.initialize();
		CCspec.initialize();

	}

	/**
	 * This method loads the ec specification from a file.
	 * 
	 * @param filename
	 *            of ec specification file
	 * @return ec specification
	 * @throws Exception
	 *             if specification could not be loaded
	 */
	private static LRSpec getLRSpecFromFile(String filename) throws Exception {
		FileInputStream inputStream = new FileInputStream(filename);
		return DeserializerUtil.deserializeLRSpec(inputStream);

	}

	private static ECSpec getECSpecFromFile(String filename) throws Exception {
		FileInputStream inputStream = new FileInputStream(filename);
		return DeserializerUtil.deserializeECSpec(inputStream);

	}

	private static CCSpec getCCSpecFromFile(String filename) throws Exception {
		FileInputStream inputStream = new FileInputStream(filename);
		return DeserializerUtil.deserializeCCSpec(inputStream);

	}

}
