/*
 * Copyright (C) 2008-2010, Aspire
 * 
 * Aspire is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License version 2.1 as published by
 * the Free Software Foundation (the "LGPL").
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library in the file COPYING-LGPL-2.1; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * 
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied. See the GNU Lesser General Public License
 * for the specific language governing rights and limitations.
 */

package org.inria.pops.readers.tagsys;

/**
 * @author Lei Zhang e-mail: lzhang@inria.fr
 *
 */

import java.util.ArrayList;

import com.sirit.data.DataManager;
import com.sirit.driver.IEventListener;
import com.sirit.mapping.EventInfo;
import com.sirit.mapping.ModemManager;
import com.sirit.mapping.ReaderManager;

public class TagsysAdaptor implements IEventListener {

	ArrayList<String> RFIDTagID = new ArrayList<String>();  // TagId in Hex without prefix"0x"
	ModemManager MM;
	DataManager dataManager;

	public void initialReader(String ReaderIP,String Antenna)throws Exception{
		
		 dataManager = new DataManager(
				DataManager.ConnectionTypes.SOCKET, ReaderIP, 0);
		dataManager.open();

		 MM = new ModemManager(dataManager);

		ReaderManager readerManager = new ReaderManager(dataManager);
		if (!readerManager.login("admin", "readeradmin"))
			throw new Exception("Login attempt failed: "
					+ readerManager.getLastErrorMessage());
		String id = dataManager.getEventChannel(this);
		if (!readerManager.eventsRegister(id, "event.tag.report"))
			throw new Exception("Failure to register for event: "
					+ readerManager.getLastErrorMessage());
		
	}
	
	
	/**
	 * Wait a moment to collect tags, the reader should firstly take
	 * sometime to discover the tags before doing the the following
	 * operations (ex.writing)
	 * 
	 * @param Reader IP
	 * @param Antenna name
	 */
	public ArrayList<String> ReturnDiscoveredTagIdList(String ReaderIP,String Antenna) throws Exception{
		initialReader( ReaderIP, Antenna);
		
		RFIDTagID.clear();
		try {
			Thread.currentThread().sleep(300);
		} catch (Exception ie) {
		}
		dataManager.close();
		
		return RFIDTagID;
	}
	
	
	/**
	 * This method defines all the operation related to the specific readers
	 * 
	 * @param Reader IP
	 * @param Antenna name
	 * @param Reader TagId: in Hex without predix "0x"
	 * @param OperationType
	 * @param fieldname defined under the <<opSpec>> in the ccspec file 
	 * @param data defined in <<dataSpec>>
	 * Return two strings: data and Opstatus whch are defined in P155 of ALE_1_1_1-standard
	 * 
	 */
	
	public String[]  DoOperation(String ReaderIP, String Antenna, String TagId, String OperationType, String fieldname, String data) throws Exception {
		
		// Return two strings: data and Opstatus whch are defined in P155 of ALE_1_1_1-standard
		String[] ReturnResults = {"", ""};
		
		initialReader( ReaderIP, Antenna);
		if (OperationType.equalsIgnoreCase("READ")) {
		}
		if (OperationType.equalsIgnoreCase("CHECK")) {
		}
		if (OperationType.equalsIgnoreCase("INITIALIZE")) {
		}
		if (OperationType.equalsIgnoreCase("ADD")) {
		}
		if (OperationType.equalsIgnoreCase("WRITE")) {
			String[] fieldName = fieldname.substring(1).split("\\.");

			// Note: Here we should add the prefix "0x" for the TagID to be compatible with the Tagsysreader's operation
			boolean result = MM.protocolIsocWrite("0x"+TagId, "",
					fieldName[0], fieldName[1], data, Antenna);
			if (result) {
				ReturnResults[0] = data;
				ReturnResults[1] = "SUCCESS";
			} else {
				ReturnResults[0] = "";
				ReturnResults[1] = "MISC_ERROR_TOTAL";
			}
		}
		if (OperationType.equalsIgnoreCase("DELETE")) {
		}
		if (OperationType.equalsIgnoreCase("PASSWORD")) {
		}
		if (OperationType.equalsIgnoreCase("KILL")) {
		}
		if (OperationType.equalsIgnoreCase("LOCK")) {
		}
		
		dataManager.close();
		
			return ReturnResults;
		
	}
	
	

	public void EventFound(Object sender, EventInfo eventInfo) {
		String tagIDwith0x = eventInfo
				.getParameter(EventInfo.EVENT_TAG_ARRIVE_PARAMS.TAG_ID);
		String tagID = tagIDwith0x.substring(2);
		int j;
		for (j = 0; j < RFIDTagID.size(); j++) {
			if (tagID.equalsIgnoreCase(RFIDTagID.get(j))) {
				break;
			}
		}
		if (j == RFIDTagID.size())
			RFIDTagID.add(tagID);
	}

}
